#!/usr/bin/perl -w -I ../../../lib

use 5.018;

use strict;

use Carp;

{ package Audio::Signal::Effect::Overdrive;

    use Moose;
    use Moose::Util::TypeConstraints;

    extends 'Audio::Signal::Effect';
    
    has 'gain' => (
        is => 'rw',
        isa => 'Num',
    );
    
    has 'colour' => (
        is => 'rw',
        isa => 'Num',
    );
   

    sub gencmd {
        my $self = shift;        
        my $ret = sprintf("%s %s %s",$self->fx,$self->gain,$self->colour);
        return($ret);
    }
}

1;
