#!/usr/bin/perl -w -I ../../../lib

use 5.018;

use strict;

use Carp;

{ package Audio::Signal::Tone::BrownNoise;

  use Moose;
  
  extends 'Audio::Signal::Tone';

  sub gencmd {
    my $self = shift;
    my $ret = sprintf("%s %s",$self->fm,$self->note);
    return($ret);
  }
}

1;
