#!/usr/bin/perl -w -I ../../../lib

use 5.018;

use strict;

use Carp;

{ package Audio::Signal::Effect::Delay;

    use Moose;
    use Moose::Util::TypeConstraints;

    extends 'Audio::Signal::Effect';
    
    has 'length' => (
        is => 'rw',
        isa => 'ArrayRef',
    );
    
    sub gencmd {
        my $self = shift;
        my $ret = sprintf("%s %s",$self->fx,@{$self->length});
        return($ret);
    }
}

1;
